<?php

/**
 * @file
 * Hooks provided by the Commerce GoCardless Client module.
 */

/**
 * Allows you to alter the details for a new mandate.
 *
 * @param array $mandate_details
 *   Array of mandate details.
 * @param int $order_id
 *   The Commerce order number.
 */
function hook_commerce_gc_client_mandate_details_alter(array &$mandate_details, $order_id) {

  $mandate_details['user']['first_name'] = 'Rob';
}

/**
 * Allows you to alter the details for a new subscription.
 *
 * @param array $subscription_details
 *   Array of subscription details.
 * @param int $line_item_id
 *   The Commerce line item.
 */
function hook_commerce_gc_client_subscription_details_alter(array &$subscription_details, $line_item_id) {

  $subscription_details['interval'] = 2;
  $subscription_details['interval_unit'] = 'monthly';
}

/**
 * Allows you to alter the initial payment date.
 *
 * @param int $initial_payment
 *   Unix time stamp for the initial payment.
 * @param int $line_item_id
 *   The Commerce line item.
 */
function hook_commerce_gc_client_initial_payment_date_alter(&$initial_payment, $line_item_id) {

  $initial_payment = strtotime('+2 weeks', $initial_payment);
}

/**
 * Allows you to alter the details for an initial payment.
 *
 * @param array $payment_details
 *   Array of payment details.
 * @param int $line_item_id
 *   The Commerce line item.
 */
function hook_commerce_gc_client_initial_payment_details_alter(array &$payment_details, $line_item_id) {

  $payment_details['amount'] = 200;
}

/**
 * Allows you to alter the details for a scheduled payment.
 *
 * @param array $payment_details
 *   Array of payment details.
 * @param int $line_item_id
 *   The Commerce line item.
 */
function hook_commerce_gc_client_scheduled_payment_details_alter(array &$payment_details, $line_item_id) {

  $payment_details['amount'] = 200;
}

/**
 * Allows you to alter the next scheduled payment date.
 *
 * @param int $next_payment
 *   Unix time stamp for the initial payment.
 * @param int $line_item_id
 *   The Commerce line item.
 */
function hook_commerce_gc_client_next_payment_date_alter(&$next_payment, $line_item_id) {

  $next_payment = strtotime('+2 weeks', $next_payment);
}

/**
 * Allows you to respond to a webhook from GoCardless.
 *
 * @param array $webhook_details
 *   Array of webhook details.
 */
function hook_commerce_gc_client_webhook(array $webhook_details) {

  if ($webhook_details['event']['resource_type'] == 'mandates') {
    $order = commerce_order_load($webhook_details['order_id']);
    commerce_order_save($order);
  }
}

/**
 * Allows you to respond to a payment creation during checkout.
 *
 * @param object $payment
 *   Object returned by GoCardless from initial payment creation.
 * @param int $line_item_id
 *   The Commerce line item.
 */
function hook_commerce_gc_client_initial_payment_created($payment, $line_item_id) {

  drupal_set_message(t("The payment's status is @status", array(
    '@status' => $payment->status,
  )));
}

/**
 * Allows you to respond to a scheduled payment creation.
 *
 * @param object $payment
 *   Object returned by GoCardless from scheduled payment creation.
 * @param int $line_item_id
 *   The Commerce line item.
 */
function hook_commerce_gc_client_scheduled_payment_created($payment, $line_item_id) {
  watchdog('commerce_gc_client', "The payment's status is @status for line item @line_item_id", array(
    '@status' => $payment->status,
    '@line_item_id' => $line_item_id,
  ));
}

/**
 * Allows you to respond to a manual payment creation.
 *
 * @param object $payment
 *   Object returned by GoCardless from initial payment creation.
 * @param int $line_item_id
 *   The Commerce line item.
 */
function hook_commerce_gc_client_manual_payment_created($payment, $line_item_id) {
  drupal_set_message(t("The payment's status is @status", array(
    '@status' => $payment->status,
  )));
}
