<?php

/**
 * @file
 * Payment method callback functions.
 */

/**
 * Payment method callback: settings form.
 */
function commerce_gc_client_settings_form($settings = NULL) {
  $form = array(
    '#prefix' => '<div id="connect-wrapper">',
    '#suffix' => '</div>',
  );

  $partner = new CommerceGcClientPartner();
  $settings = $partner->settings;
  $sandbox = $settings['connect']['sandbox'];
  $sandbox ? $ext = '_sandbox' : $ext = '_live';

  $connected = FALSE;
  if ($partner->settings['org_id']) {
    $authenticate = $partner->api(array());
    if ($authenticate == 200) {
      $connected = TRUE;
    }
  }

  // Set message if returning from Partner site after successful connection.
  if ($connected && variable_get('commerce_gc_client_connecting')) {
    drupal_set_message(t('You have connected successfully with GoCardless as a client of Seamless CMS'));
    variable_del('commerce_gc_client_connecting');
  }

  $form['connect'] = array(
    '#type' => 'fieldset',
    '#title' => t('Connect with GoCardless'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
  );

  $form['connect']['sandbox'] = array(
    '#type' => 'checkbox',
    '#title' => '<b>' . t('Enable Sandbox') . '</b>',
    '#description' => t('Sandbox: GoCardless will operate in a test environment, and no real banking activity will occur.'),
    '#default_value' => $sandbox,
    '#ajax' => array(
      'callback' => 'commerce_gc_client_sandbox_callback',
      'wrapper' => 'connect-wrapper',
      'method' => 'replace',
    ),
  );

  $markup = '<br /><p><b>' . t('Connect / disconnect with GoCardless') . '</b></p>';

  if (!$connected) {
    $markup .= '<p>' . t('After clicking "Connect" you will be redirected to the GoCardless where you can create an account and connect your site as a client of Seamless-CMS.co.uk') . '</p>';
    $form['connect']['markup'] = array(
      '#markup' => $markup,
    );

    $form['connect']['connect'] = array(
      '#type' => 'submit',
      '#disabled' => !$sandbox && !isset($_SERVER['HTTPS']) ? TRUE : FALSE,
      '#value' => $sandbox ? t('Connect SANDBOX') : t('Connect LIVE'),
      '#submit' => array('commerce_gc_client_connect_submit'),
      '#suffix' => !$sandbox && !isset($_SERVER['HTTPS']) ? '<br /><i>' . t('Site needs to be secure (https) before you can connect to GoCardless LIVE.') . '</i>' : NULL,
    );

    $form['connect']['path'] = array(
      '#type' => 'value',
      '#value' => current_path(),
    );
  }
  else {
    $form['connect']['markup'] = array(
      '#markup' => $markup,
    );

    $confirm = t('Are you sure you want to disconnect your site from GoCardless?');
    $form['connect']['disconnect'] = array(
      '#type' => 'submit',
      '#value' => $sandbox ? t('Disconnect SANDBOX') : t('Disconnect LIVE'),
      '#submit' => array('commerce_gc_client_disconnect_submit'),
      '#attributes' => array(
        'onclick' => 'if (!confirm("' . $confirm . '")) {return false;}',
      ),
    );
  }

  global $base_url;
  $webhook_url = $base_url . '/gc_client/webhook';

  $gc_webhook_url = t('https://manage@env.gocardless.com/developers/webhook-endpoints/create', array(
    '@env' => $sandbox ? '-sandbox' : '',
  ));

  $webhook_prefix = t('To receive webhooks, add <i>@webhook_url</i> as webhook URL and set the secret to the same as the Webhook Secret field from <a target="new" href="@gc_webhook_url">here</a>.', array(
    '@webhook_url' => $webhook_url,
    '@gc_webhook_url' => $gc_webhook_url,
  )) . '<br /><br />';

  $form['connect']['webhook' . $ext] = array(
    '#type' => 'textfield',
    '#title' => t('Webhook secret'),
    '#default_value' => $settings['webhook'],
    '#field_prefix' => $webhook_prefix,
  );

  // Save live secret if $sandbox = TRUE, and vice versa.
  $sandbox ? $webhook_suffixed = 'webhook_live' : $webhook_suffixed = 'webhook_sandbox';
  $form['connect'][$webhook_suffixed] = array(
    '#type' => 'value',
    '#value' => isset($settings['connect'][$webhook_suffixed]) ? $settings['connect'][$webhook_suffixed] : NULL,
  );

  // Global.
  $form['global'] = array(
    '#type' => 'fieldset',
    '#title' => t('General settings'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
  );

  $form['global']['payments_tab'] = array(
    '#type' => 'checkbox',
    '#title' => t('Hide Payments tab'),
    '#default_value' => $settings['global']['payments_tab'],
    '#description' => t("Commerce Payments do not work very well with GoCardless yet, so checking this will hide the Payments tab when viewing any orders created with this payment method."),
  );

  $form['global']['currencies'] = array(
    '#type' => 'checkbox',
    '#title' => t('Allow customer to choose currency at GoCardless'),
    '#default_value' => $settings['global']['currencies'],
    '#description' => t('When directed to GoCardless during checkout, the customer is limited to the default Commerce currency, or the currency of the produce in their cart. If this option is enabled then the customer will be able to choose from all the currencies that you have enabled at GoCardless for their new direct debit mandate. See <a href = "@path">here</a> for more options, and information on enabling multiple currencies with GoCardless.', array(
      '@path' => '/admin/commerce/config/currency/gocardless',
    )),
  );

  $form['global']['fixer'] = array(
    '#type' => 'textfield',
    '#title' => t('fixer.io API KEY'),
    '#default_value' => $settings['global']['fixer'],
    '#description' => t("If you provide an API key, and the customer has chosen to set up their direct debit using a differnt currency, then during payment creation the amount will be automaticically calculated using current exchange rates at <a target='new' href='@url'>fixer.io</a>. If you do not provide an API key payments will be the standard amount for the products the customer has purchased, regardless of the currency that the direct debit mandate was created to use.", array(
      '@url' => 'http://fixer.io',
    )),
    '#size' => 40,
    '#maxlength' => 40,
    '#states' => array(
      'visible' => array(
        'input[name="parameter[payment_method][settings][payment_method][settings][global][currencies]"]' => array('checked' => TRUE),
      ),
    ),
  );

  $form['global']['emails'] = array(
    '#type' => 'textfield',
    '#title' => t('Email'),
    '#default_value' => variable_get('commerce_gc_client_warnings_email'),
    '#default_value' => $settings['global']['emails'],
    '#description' => t("Email address to send warnings."),
    '#size' => 40,
    '#maxlength' => 40,
  );

  $form['global']['payment_limit'] = array(
    '#type' => 'textfield',
    '#title' => t('Maximum payments'),
    '#default_value' => $settings['global']['payment_limit'],
    '#size' => 3,
    '#maxlength' => 3,
    '#description' => t("The maximum number of payments that can be raised automatically, per order, per day. If the amount is exceeded, a warning email is sent to the specified address above. Leave unset for unlimitted."),
    '#element_validate' => array('element_validate_integer_positive'),
    '#required' => FALSE,
  );

  $form['global']['checkout_label'] = array(
    '#type' => 'textfield',
    '#title' => t('Checkout button label'),
    '#description' => t('Provide customised value for checkout button. (This only works if GoCardless is the only enabled payment method.)'),
    '#default_value' => $settings['global']['checkout_label'],
  );

  // Debugging options.
  $form['log'] = array(
    '#type' => 'fieldset',
    '#title' => t('Logging'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
  );

  $form['log']['log_webhook'] = array(
    '#prefix' => '<p>',
    '#suffix' => '</p>',
    '#type' => 'checkbox',
    '#title' => '<b>' . t('Enable webhook logging') . '</b>',
    '#description' => t('Webhooks recieved from GoCardless will be written to the log.'),
    '#default_value' => $settings['log']['log_webhook'],
  );

  $form['log']['log_api'] = array(
    '#prefix' => '<p>',
    '#suffix' => '</p>',
    '#type' => 'checkbox',
    '#title' => '<b>' . t('Enable API logging') . '</b>',
    '#description' => t('Responses from the Partner site to API posts will be written to the log.'),
    '#default_value' => $settings['log']['log_api'],
  );
  return $form;
}

/**
 * Ajax callback: Refreshes settings page after toggling the sandbox option.
 */
function commerce_gc_client_sandbox_callback(&$form, &$form_state) {
  $connect = $form_state['values']['parameter']['payment_method']['settings']['payment_method']['settings']['connect'];
  $rule = rules_config_load('commerce_payment_gocardless_client');
  foreach ($rule->actions() as $action) {
    if (is_callable(array($action, 'getElementName')) && $action->getElementName() == 'commerce_payment_enable_gocardless_client') {
      if (is_array($action->settings['payment_method']) && !empty($action->settings['payment_method']['settings'])) {
        $action->settings['payment_method']['settings']['connect'] = $connect;
      }
      else {
        $action->settings['payment_method'] = array(
          'method_id' => 'gocardless_client',
          'settings' => array(
            'connect' => $connect,
          ),
        );
      }
    }
  }
  $action->save();

  ctools_include('ajax');
  ctools_add_js('ajax-responder');
  $commands[] = ctools_ajax_command_reload();
  print ajax_render($commands);
  drupal_exit();
}

/**
 * Payment method callback: redirect form.
 */
function commerce_gc_client_redirect_form($form, &$form_state, $order, $payment_method) {
  $order_wrapper = entity_metadata_wrapper('commerce_order', $order);
  $address = $order_wrapper->commerce_customer_billing->commerce_customer_address->value();
  $settings = CommerceGcClientPartner::getSettings();
  $scheme = NULL;
  if (!$settings['global']['currencies']) {
    $currency_code = $order_wrapper->commerce_order_total->value()['currency_code'];
    foreach (variable_get('commerce_gc_client_currency_schemes') as $currency_scheme => $value) {
      if ($currency_code == $currency_scheme) {
        $scheme = $value['scheme'];
        break;
      }
    }
  }
  $complete_path = '/checkout/' . $order->order_number . '/payment/return/' . $order->data['payment_redirect_key'] . '/';
  $redirect_url = $settings['partner_url'] . '/gc_partner/mandate?payment_method=gocardless_client&org_id=' . $settings['org_id'] . '&order_id=' . $order->order_number . '&complete_path=' . $complete_path;

  // Generate the mandate_details array.
  $mandate_details = array(
    'name' => 'Order ' . $order->order_number,
    'scheme' => $scheme,
    'redirect_url' => $redirect_url,
    'user' => array(
      'first_name' => $address['first_name'],
      'last_name' => $address['last_name'],
      'email' => $order->mail,
      'company_name' => $address['organisation_name'],
      'billing_address1' => $address['thoroughfare'],
      'billing_address2' => $address['premise'],
      'billing_town' => $address['locality'],
      'billing_postcode' => $address['postal_code'],
      'billing_country' => $address['country'],
    ),
  );

  // Provide a hook so that mandate details can be altered by another module.
  drupal_alter('commerce_gc_client_mandate_details', $mandate_details, $order->order_number);

  $partner = new CommerceGcClientPartner();
  $result = $partner->api(array(
    'endpoint' => 'redirect_flows',
    'action' => 'create',
    'mandate_details' => $mandate_details,
    'order_id' => $order->order_number,
  ));

  if (valid_url($result->response)) {

    drupal_goto($result->response);
  }
  else {
    drupal_set_message(t('Something went wrong during connection with GoCardless'), 'error');
  }
  return $form;
}

/**
 * Payment method callback: redirect form return validation.
 *
 * @TODO Validation should take place before mandate is created.
 */
function commerce_gc_client_redirect_form_validate($order, $payment_method) {
  // If for some reason the payment redirect key in this return path does not
  // match that in the order, prevent processing.
  if (arg(4) != $order->data['payment_redirect_key']) {
    watchdog('commerce_gc_client', 'Customer returned from GoCardless with a non-matching redirect key.', array(), WATCHDOG_WARNING);
    return FALSE;
  }
}

/**
 * Payment method callback: redirect form return submission.
 */
function commerce_gc_client_redirect_form_submit($order, $payment_method) {
  $redirect_id = arg(5);
  $mandate_id = arg(6);
  $customer_id = arg(7);
  $order_number = arg(8);

  $partner = new CommerceGcClientPartner();
  $result = $partner->api([
    'endpoint' => 'mandates',
    'action' => 'get',
    'mandate' => $mandate_id,
  ]
  );
  if ($result->response->status_code != 200) {
    // TODO do something here.
  }
  else {
    $mandate = $result->response->body->mandates;
  }

  // Insert info about the order into the database.
  $gcid = db_insert('commerce_gc_client')->fields(array(
    'order_id' => $order_number,
    'gc_mandate_id' => $mandate_id,
    'gc_mandate_status' => 'created',
    'gc_mandate_scheme' => $mandate->scheme,
    'created' => REQUEST_TIME,
    'data' => serialize(array(
      'gc_redirect_id' => $redirect_id,
      'gc_customer_id' => $customer_id,
    )),
    'sandbox' => $payment_method['settings']['connect']['sandbox'],
  ))->execute();

  $comment = t('Your new direct debit mandate @mandate has been created by GoCardless.', array(
    '@mandate' => $mandate_id,
  ));
  drupal_set_message($comment);

  $order_wrapper = entity_metadata_wrapper('commerce_order', $order);
  module_load_include('inc', 'commerce_gc_client', 'commerce_gc_client.calculate');

  // Set up seperate payments / subscriptions for each line item in order.
  foreach ($order_wrapper->commerce_line_items as $line_item_wrapper) {

    $line_item = $line_item_wrapper->value();
    $line_item_id = $line_item->line_item_id;
    if ($line_item->type != 'product') {
      continue;
    }

    if (!isset($line_item->data['gc'])) {

      $data = array();
    }
    elseif (!$line_item->data['gc']['gc_use']) {
      $data = array();
    }
    else {
      $data = $line_item_wrapper->value()->data['gc'];
    }
    if (empty($data)) {
      $data['type'] = 'P';
    }

    // Payment type is subscription.
    if ($data['type'] == 'S') {

      $calculate = commerce_gc_client_price_calculate($order, $line_item_wrapper, 'new');

      $subscription_details = array(
        'endpoint' => 'subscriptions',
        'action' => 'create',
        'mandate' => $mandate_id,
        'amount' => $calculate['amount'] / 100,
        'currency' => $calculate['currency_code'],
        'name' => t('Subscription for @title', array(
          '@title' => $line_item_wrapper->commerce_product->title->value(),
        )),
        'interval' => $data['interval_params']['length'],
        'interval_unit' => $data['interval_params']['unit'],
        'start_date' => isset($data['start_date']) ? date('Y-m-d', $data['start_date']) : NULL,
        'count' => isset($data['count']) ? $data['count'] : NULL,
        'end_date' => isset($data['end_date']) ? date('Y-m-d', $data['end_date']) : NULL,
        'day_of_month' => isset($data['dom']) ? $data['dom'] : NULL,
        'month' => isset($data['month']) ? $data['month'] : NULL,
        'metadata' => array(
          'line_item_id' => $line_item_id,
        ),
      );

      // Provide hook so that subscription details can be altered by another
      // module.
      drupal_alter('commerce_gc_client_subscription_details', $subscription_details, $line_item_id);
      $partner = new CommerceGcClientPartner();
      $result = $partner->api($subscription_details);

      if ($result->response->status_code == 201) {
        $subscription = $result->response->body->subscriptions;
        $comment_arr = array(
          '@product' => $line_item_wrapper->commerce_product->title->value(),
          '@interval' => $subscription->interval,
          '@interval_unit' => $subscription->interval_unit,
          '@amount' => commerce_currency_format($subscription->amount, $calculate['currency_code']),
          '@first_payment' => format_date(strtotime(array_shift($subscription->upcoming_payments)->charge_date), 'gocardless_client'),
        );

        $comment = t('Your @interval @interval_unit subscription of @amount for <b>@product</b> has been created with GoCardless, and the first payment will be made from your bank on @first_payment.', $comment_arr);
        drupal_set_message($comment);

        db_insert('commerce_gc_client_item')->fields(array(
          'gcid' => $gcid,
          'line_item_id' => $line_item_id,
          'type' => $data['type'],
          'gc_subscription_id' => $subscription->id,
          'gc_subscription_status' => 'created',
        ))->execute();
      }
      else {
        drupal_set_message(t('Something went wrong creating your subscription with GoCardless. Please contact the site administrator for assistance.'), 'warning');
      }
    }

    // Payment type is payment.
    elseif ($data['type'] == 'P') {

      db_insert('commerce_gc_client_item')->fields(array(
        'line_item_id' => $line_item_id,
        'gcid' => $gcid,
        'type' => $data['type'],
      ))->execute();

      if (count($data) == 1) {

        $first_payment = 1;
      }
      else {
        $first_payment = commerce_gc_client_start_date_calculate($data);
      }

      // Provide hook so initial date can be modified by another module.
      drupal_alter('commerce_gc_client_initial_payment_date', $first_payment, $line_item_id);

      // If the product is set to create the first payment immediately.
      if ($first_payment == 1) {

        $calculate = commerce_gc_client_price_calculate($order, $line_item_wrapper, 'new');

        $initial_payment = commerce_currency_format($calculate['amount'], $calculate['currency_code']);
        $order->data['initial_payment'] = $initial_payment;

        $payment_details = array(
          'endpoint' => 'payments',
          'action' => 'create',
          'mandate' => $mandate_id,
          'amount' => $calculate['amount'] / 100,
          'currency' => $calculate['currency_code'],
          'description' => t('Payment for @title', array(
            '@title' => $line_item_wrapper->commerce_product->title->value(),
          )),
          'metadata' => array(
            'line_item_id' => $line_item_id,
          ),
        );

        // Provide hook so that payment details can be altered by another
        // module.
        drupal_alter('commerce_gc_client_initial_payment_details', $payment_details, $line_item_id);

        $partner = new CommerceGcClientPartner();
        $result = $partner->api($payment_details);

        if ($result->response->status_code == 201) {

          $payment = $result->response->body->payments;

          // Update next_payment field in gc_client table.
          if (isset($data['interval_params'])) {
            $next_payment = strtotime('+' . $data['interval_params']['string']);
          }

          // Let everyone know what is going on.
          $comment_array = array(
            '@amount' => commerce_currency_format($payment->amount, $calculate['currency_code']),
            '@charge_date' => format_date(strtotime($result->response->body->payments->charge_date), 'gocardless_client'),
            '@product' => $line_item_wrapper->commerce_product->title->value(),
          );
          $comment = t('A payment of @amount for <b>@product</b> has been created and will be made from your bank on @charge_date.', $comment_array);
          drupal_set_message($comment);

          // Provide a hook so that other modules can respond to payment
          // creation.
          module_invoke_all('commerce_gc_client_initial_payment_created', $payment, $line_item_id);
        }
        else {
          drupal_set_message(t('There was a problem creating your initial payment. We will try to create it again later.'), 'warning');

          // Update next_payment to current time so that it will get picked
          // up on next cron run.
          $next_payment = REQUEST_TIME;
        }
      }

      // Else if a start date is set for the product then defer the first
      // payment creation.
      elseif ($first_payment) {
        $next_payment = $first_payment;
        $calculate = commerce_gc_client_price_calculate($order, $line_item_wrapper);

        $initial_payment = commerce_currency_format($calculate['amount'], $calculate['currency_code']);
        $order->data['initial_payment'] = $initial_payment;
        $comment = t('A payment for @initial_payment will be created with GoCardless on @charge_date.', array(
          '@initial_payment' => $initial_payment,
          '@charge_date' => format_date(strtotime('midnight', $first_payment), 'gocardless_client'),
        ));
        drupal_set_message($comment);
      }

      if (isset($next_payment)) {
        db_update('commerce_gc_client_item')->fields(array(
          'next_payment' => $next_payment,
        ))->condition('line_item_id', $line_item_id)->execute();
      }
    }
    $line_item_wrapper->save();
  }
  $order_wrapper->save();
}
