<?php

/**
 * @file
 * Hooks that must only be available when the payment method is active.
 */

/**
 * Implements hook_field_widget_field_alter().
 *
 * Filters for available countries / currencies.
 */
function commerce_gc_client_field_widget_form_alter(&$element, &$form_state, $context) {
  // Filters the list of currencies available in the Product UI form if
  // 'Create payments in foreign currencies' is enabled, and the product is
  // set to use  GoCardless recurrence rules.
  if ($form_state['build_info']['form_id'] == 'commerce_product_ui_product_form' && $element['#field_name'] == 'commerce_price' && isset($element['currency_code']['#options']) && isset($element['#entity']->data['gc'])) {
    $data = $element['#entity']->data['gc'];
    $settings = commerce_gc_client_load_settings();
    if ($data['gc_use'] && !$settings['global']['currencies']) {
      $options = $element['currency_code']['#options'];
      $gc_currencies = variable_get('commerce_gc_client_currency_schemes');
      foreach ($options as $currency_code) {
        if (!$gc_currencies[$currency_code]['enabled']) {
          unset($options[$currency_code]);
        }
      }
      $default = variable_get('commerce_default_currency');
      $options = array($default => $default) + $options;
      $element['currency_code']['#options'] = $options;
    }
  }
  // Filters the available countries list in the checkout form if
  // 'Allow customer to choose currency at GoCadless' is disabled, and
  // there is a GoCardless product in the cart.
  if (arg(0) == 'checkout' && is_numeric(arg(1)) && $context['instance']['bundle'] == 'billing') {
    $options = commerce_gc_client_addressfield_country_options($element);
    if (($options)) {
      $element['country']['#options'] = $options;
    }
  }
}

/**
 * Implements hook_addressfield_standard_widget_refesh_alter().
 *
 * Overrides available countries after the country has been refreshed in
 * commerce_customer_address fields.
 */
function commerce_gc_client_addressfield_standard_widget_refresh_alter(&$commands, $form, $form_state) {
  $array_parents = $form_state['triggering_element']['#array_parents'];
  array_pop($array_parents);
  $element = $form;
  foreach ($array_parents as $name) {
    $element = &$element[$name];
    if (!empty($element['#addressfield'])) {
      break;
    }
  }
  if ($element['#field_name'] != 'commerce_customer_address' && $element['#bundle'] != 'billing') {
    return;
  }
  $options = commerce_gc_client_addressfield_country_options($element);
  if ($options) {
    // Return the address block, but remove the '_weight' element inserted
    // by the field API, and update the country options.
    unset($element['_weight']);
    $element['country']['#options'] = $options;
    $commands[0] = ajax_command_replace(NULL, render($element));
  }
}

/**
 * Implements hook_addressfield_default_values_alter().
 *
 * Sets the default country to match the currency of cart objects but only
 * if there is one country for the currency, and the cart products use
 * GoCardless.
 */
function commerce_gc_client_addressfield_default_values_alter(&$default_values, $context) {
  if (arg(0) == 'checkout' && is_numeric(arg(1)) && $context['instance']['bundle'] == 'billing') {
    if ($currency_code = commerce_gc_client_cart_product_currency()) {
      $currencies = variable_get('commerce_gc_client_currency_schemes');
      if (count($currencies[$currency_code]['countries'] == 1)) {
        $default_values['country'] = key($currencies[$currency_code]['countries']);
      }
    }
  }
}

/**
 * Implements hook_form_FORM_ID_alter() for commerce_checkout_form_review().
 *
 * Overrides the submit button on the checkout review form if GoCardless
 * is the only loaded payment method.
 */
function commerce_gc_client_form_commerce_checkout_form_review_alter(&$form, $form_state, $form_id) {
  if (count($methods = $form['commerce_payment']['payment_methods']['#value']) == 1) {
    if (reset($methods)['method_id'] == 'gocardless_client') {
      $settings = commerce_gc_client_load_settings();
      $form['buttons']['continue']['#value'] = $settings['global']['checkout_label'];
    }
  }
}
