<?php

/**
 * @file
 * Additional configuration options for Commerce product forms.
 */

/**
 * Implements hook_form_FORM_ID_alter() for commerce_product_ui_product_form().
 *
 * Add extra GoCardless recurrence rules settings to Commerce product forms.
 */
function commerce_gc_client_form_commerce_product_ui_product_form_alter(&$form, $form_state, $form_id) {
  $gc_product_types = variable_get('commerce_gc_client_product_types');
  if (!isset($gc_product_types[$form['#bundle']])) {
    return;
  }

  if (module_exists('date_popup')) {
    drupal_add_css(drupal_get_path('module', 'commerce_gc_client'));
  }

  $defaults = NULL;
  if (isset($form['#entity']->data)) {
    if (isset($form['#entity']->data['gc'])) {
      $defaults = $form['#entity']->data['gc'];
    }
  }

  $form['#validate'][] = 'commerce_gc_client_product_form_validate';

  foreach (array(
    'gc_use' => FALSE,
    'type' => 'P',
    'create_payment' => FALSE,
    'count' => NULL,
    'start_date' => NULL,
    'end_date' => NULL,
    'interval_selector' => 0,
    'interval_length' => NULL,
    'interval_unit' => NULL,
    'payment_int' => NULL,
    'dom' => NULL,
    'month' => NULL,
  ) as $key => $value) {
    if (isset($form_state['values'])) {
      if (in_array($key, array('start_date', 'end_date'))) {
        if (is_array($form_state['values'][$key])) {
          if (!empty($form_state['values'][$key]['date'])) {
            $$key = date('Y-m-d', strtotime($form_state['values'][$key]['date']));
          }
          else {
            $$key = $value;
          }
        }
        elseif (isset($form_state['values'][$key])) {
          $$key = date('Y-m-d', strtotime($form_state['values'][$key]));
        }
        else {
          $$key = $value;
        }
      }
      else {
        $form_state['values'][$key] ? $$key = $form_state['values'][$key] : $$key = $value;
      }
    }
    elseif (isset($defaults[$key])) {
      if (in_array($key, array('start_date', 'end_date'))) {
        $$key = date('Y-m-d', $defaults[$key]);
      }
      else {
        $$key = $defaults[$key];
      }
    }
    else {
      $$key = $value;
    }
  }

  $ajax = array(
    'callback' => 'commerce_gc_client_product_use_settings_ajax',
    'wrapper' => 'gc-client-product-use-settings-ajax',
    'method' => 'replace',
  );

  $dom_options = array();
  for ($d = -1; $d <= 28; $d++) {
    if ($d == -1) {
      $dom_options[$d] = t('Last day of month');
    }
    elseif ($d == 0) {
      continue;
    }
    else {
      $dom_options[$d] = $d;
    }
  }

  $month_options = array();
  for ($m = 1; $m <= 12; $m++) {
    $month = t('@month', array('@month' => date('F', mktime(0, 0, 0, $m, 1))));
    $month_options[strtolower($month)] = $month;
  }

  $dom_access = FALSE;
  $dom_required = FALSE;
  $month_access = FALSE;
  if ($type == 'S' || ($type == 'P' && !$create_payment)) {
    if ($interval_selector) {
      $int_arr = explode(' ', trim($payment_interval));
      if (count($int_arr) == 2) {
        if (in_array($int_arr[1], array('monthly', 'yearly'))) {
          $dom_access = TRUE;
        }
        if ($int_arr[1] == 'yearly') {
          $month_access = TRUE;
        }
        if ($int_arr[1] == 'monthly') {
          $dom_required = TRUE;
        }
      }
    }
    else {
      if (in_array($interval_unit, array('monthly', 'yearly'))) {
        $dom_access = TRUE;
      }
      if ($interval_unit == 'yearly') {
        $month_access = TRUE;
      }
      if ($interval_unit == 'monthly') {
        $dom_required = TRUE;
      }
    }
  }

  $form['gc'] = array(
    '#type' => 'fieldset',
    '#title' => t('GoCardless recurrence settings'),
    '#collapsible' => TRUE,
    '#collapsed' => $gc_use ? FALSE : TRUE,
    '#weight' => 30,
    '#prefix' => '<div id="gc-client-product-use-settings-ajax">',
    '#suffix' => '</div>',
  );

  $form['gc']['gc_use'] = array(
    '#type' => 'checkbox',
    '#default_value' => $gc_use,
    '#title' => t("Use GoCardless recurrence rules with this product?"),
    '#ajax' => $ajax,
  );

  $form['gc']['gc_use_container'] = array(
    '#type' => 'container',
    '#collapsible' => FALSE,
    '#access' => $gc_use,
  );

  $form['gc']['gc_use_container']['type'] = array(
    '#type' => 'radios',
    '#title' => t('Payments type'),
    '#default_value' => $type,
    '#options' => array(
      'P' => t('One-off payments'),
      'S' => t('Subscription'),
    ),
    '#description' => t('Subscription payments created automatically by GoCardless, or one-off payments created by your website.'),
    '#weight' => 0,
    '#ajax' => $ajax,
  );

  $form['gc']['gc_use_container']['create_payment'] = array(
    '#type' => 'checkbox',
    '#title' => '<b>' . t('Create payment immediately') . '</b>',
    '#default_value' => $create_payment,
    '#description' => t("Automatically create a payment upon completeion of checkout."),
    '#ajax' => $ajax,
    '#access' => $type == 'P' ? TRUE : FALSE,
  );

  $form['gc']['gc_use_container']['start_date'] = array(
    '#title' => t('Start date'),
    '#type' => module_exists('date_popup') ? 'date_popup' : 'date_select',
    '#date_format' => 'd M Y',
    '#datepicker_options' => array('minDate' => 0),
    '#date_year_range' => '0:+1',
    '#default_value' => $start_date,
    '#description' => t("The date on which the first payment should be charged. Must be within one year of creation."),
    '#access' => $type == 'S' || ($type == 'P' && !$create_payment) ? TRUE : FALSE,
  );

  $form['gc']['gc_use_container']['count'] = array(
    '#title' => t('Count'),
    '#type' => 'textfield',
    '#default_value' => $count,
    '#element_validate' => array('element_validate_integer_positive'),
    '#size' => 5,
    '#maxlength' => 5,
    '#description' => t("The total number of payments that should be taken by this subscription."),
    '#ajax' => $ajax,
    '#access' => $type == 'S' && !isset($end_date) ? TRUE : FALSE,
  );

  $form['gc']['gc_use_container']['end_date'] = array(
    '#title' => t('End date'),
    '#type' => module_exists('date_popup') ? 'date_popup' : 'date_select',
    '#date_format' => 'd M Y',
    '#datepicker_options' => array('minDate' => 0),
    '#default_value' => $end_date,
    '#description' => t("Date on or after which no further payments should be created. If this field is blank and count is not specified, the subscription will continue until it is cancelled."),
    '#suffix' => '<br />',
    '#ajax' => $ajax,
    '#access' => $type == 'S' && (!isset($count) || empty($count)) ? TRUE : FALSE,
  );

  $form['gc']['gc_use_container']['interval_length'] = array(
    '#type' => 'textfield',
    '#title' => t('Interval length between payments'),
    '#default_value' => $interval_length,
    '#size' => 3,
    '#maxlength' => 3,
    '#description' => t("Number of interval units between customer charge dates. Must be greater than or equal to 1. Must result in at least one charge date per year. Defaults to 1 for subscription payments."),
    '#element_validate' => array('element_validate_integer_positive'),
    '#weight' => 2,
    '#ajax' => $ajax,
  );

  $form['gc']['gc_use_container']['interval_unit'] = array(
    '#type' => 'select',
    '#title' => t('Interval unit between payments'),
    '#default_value' => $interval_unit,
    '#options' => array(
      'weekly' => t('Week'),
      'monthly' => t('Month'),
      'yearly' => t('Year'),
    ),
    "#empty_option" => t('- None -'),
    '#description' => t("The unit of time between customer charge dates."),
    '#weight' => 2,
    '#ajax' => $ajax,
  );

  $form['gc']['gc_use_container']['markup_container'] = array(
    '#type' => 'container',
    '#collapsible' => FALSE,
    '#weight' => 2,
  );

  $form['gc']['gc_use_container']['dom'] = array(
    '#type' => 'select',
    '#title' => t('Day of month'),
    '#options' => $dom_options,
    '#default_value' => $dom,
    '#weight' => 2,
    '#description' => t("The day of the month to charge customers on. (As per RFC 2445.)"),
    '#empty_option' => '- ' . t('select') . ' -',
    '#access' => $dom_access,
    '#ajax' => $ajax,
    '#required' => $month || $dom_required ? TRUE : FALSE,
  );

  $form['gc']['gc_use_container']['month'] = array(
    '#type' => 'select',
    '#title' => t('Month'),
    '#options' => $month_options,
    '#default_value' => $month,
    '#description' => t("Name of the month on which to charge a customer."),
    '#weight' => 2,
    '#empty_option' => '- ' . t('select') . ' -',
    '#access' => $month_access ? TRUE : FALSE,
    '#required' => $dom ? TRUE : FALSE,
    '#ajax' => $ajax,
  );

  if (isset($form['gocardless_payment_int'])) {

    $form['gc']['gc_use_container']['interval_selector'] = array(
      '#type' => 'checkbox',
      '#prefix' => '<b>' . t('Interval selection mode') . '</b>',
      '#title' => t('Allow customers to select payment interval using product attributes.'),
      '#default_value' => $interval_selector,
      '#ajax' => $ajax,
    );

    $form['gc']['gc_use_container']['payment_int'] = array(
      '#type' => 'select',
      '#title' => t('Payment interval'),
      '#options' => $form['gocardless_payment_int'][LANGUAGE_NONE]['#options'],
      '#default_value' => isset($payment_interval) ? $payment_interval : NULL,
      '#weight' => 1,
      '#access' => $interval_selector ? TRUE : FALSE,
      '#required' => $interval_selector ? TRUE : FALSE,
      '#ajax' => $ajax,
    );

    unset($form['gocardless_payment_int']);

    $form['gc']['gc_use_container']['interval_length']['#access'] = $interval_selector ? FALSE : TRUE;
    $form['gc']['gc_use_container']['interval_length']['#required'] = $type == 'S' || ($type == 'P' && $interval_unit) ? TRUE : FALSE;

    $form['gc']['gc_use_container']['interval_unit']['#access'] = $interval_selector ? FALSE : TRUE;
    $form['gc']['gc_use_container']['interval_unit']['#required'] = $type == 'S' || ($type == 'P' && $interval_length) ? TRUE : FALSE;
  }
}

/**
 * Ajax callback function; returns payment method section of form.
 *
 * @see commerce_product_ui_product_form()
 */
function commerce_gc_client_product_use_settings_ajax(&$form, &$form_state) {
  return $form['gc'];
}

/**
 * Adds additional data to the commerce_product entity.
 *
 * For some reason this does not work with a custom form submission handler,
 * when saving products, so a form validation handler is used instead.
 *
 * @see commerce_gc_client_product_form_validate()
 */
function _commerce_gc_client_product_form_validate(&$form, &$form_state) {
  if (!isset($form_state['commerce_product']->data)) {
    $form_state['commerce_product']->data = array();
  }

  if (!$form_state['values']['gc_use']) {
    if (isset($form_state['commerce_product']->data['gc'])) {
      $fields = $form_state['commerce_product']->data['gc'];
      $fields['gc_use'] = FALSE;
      $form_state['commerce_product']->data['gc'] = $fields;
    }
    return;
  }

  $payment_interval = $form_state['values']['payment_int'];
  if (!empty($payment_interval) && $payment_interval != '_none') {
    $payment_interval = explode(" ", trim($payment_interval));
    if (count($payment_interval) != 2) {
      form_set_error('gocardless_payment_int', t("There is a problem with the Payment Interval key. Please review the 'GoCardless payment interval' field settings."));
    }
    elseif (!in_array($payment_interval[1], array('weekly', 'monthly', 'yearly'))) {
      form_set_error('gocardless_payment_int', t("'@value' is not a valid interval unit. Please review the 'GoCardless payment interval' field settings.", array(
        '@value' => $payment_interval[1],
      )));
    }
  }

  $keys = array(
    'gc_use',
    'type',
    'create_payment',
    'start_date',
    'count',
    'end_date',
    'interval_selector',
    'interval_length',
    'interval_unit',
    'payment_int',
    'dom',
    'month',
  );
  $fields = array();
  foreach ($form_state['values'] as $key => $value) {
    if (!in_array($key, $keys)) {
      continue;
    }
    if ($key != 'gc_use' && isset($form['gc']['gc_use_container'][$key]['#access'])) {
      if (!$form['gc']['gc_use_container'][$key]['#access']) {
        $value = NULL;
      }
    }
    if (isset($value)) {
      if (in_array($key, array('start_date', 'end_date'))) {
        $fields[$key] = is_string($value) ? strtotime($value) : NULL;
      }
      else {
        $fields[$key] = $value;
      }
    }
    else {
      $fields[$key] = NULL;
    }
  }

  // This works if #ajax is used in product form.
  $form_state['commerce_product']->data['gc'] = $fields;

  // .. and this works if #ajax is not used in product form
  // $form['#entity']->data['gc'] = $fields;.
}
