<?php

/**
 * @file
 * Default rules configurations for module.
 */

/**
 * Implements hook_rules_defaults().
 */
function commerce_gc_client_default_rules_configuration() {
  $configs = array();
  $rules = array();

  $rules['commerce_gc_client_cart_validate'] = '{ "commerce_gc_client_cart_validate" : {
    "LABEL" : "Validate GoCardless cart items",
    "PLUGIN" : "reaction rule",
    "OWNER" : "rules",
    "REQUIRES" : [ "commerce_gc_client", "commerce_cart" ],
    "ON" : { "commerce_cart_product_prepare" : [] },
    "IF" : [
      { "commerce_gc_client_rules_products_same_currency" : {
          "commerce_order" : [ "commerce_order" ],
          "commerce_product" : [ "commerce_product" ]
        }
      }
    ],
    "DO" : [ { "commerce_gc_client_rules_add_to_cart_prevent" : [] } ]
  }}';

  if (module_exists('commerce_shipping')) {

    $rules['commerce_gc_client_calculate_field_shipping_cost'] = '{ "commerce_gc_client_calculate_field_shipping_cost" : {
    "LABEL" : "Calculate shipping using field_shipping_cost",
    "PLUGIN" : "rule",
    "OWNER" : "rules",
    "REQUIRES" : [ "rules" ],
    "USES VARIABLES" : {
      "commerce_line_item" : { "label" : "Commerce line item", "type" : "commerce_line_item" },
      "shipping_cost_for_line_item" : {
        "label" : "Shipping cost for line item",
        "type" : "decimal",
        "parameter" : false
      }
    },
    "IF" : [
      { "entity_has_field" : { "entity" : [ "commerce-line-item" ], "field" : "commerce_product" } },
      { "entity_has_field" : {
          "entity" : [ "commerce-line-item:commerce-product" ],
          "field" : "field_gocardless_shipping_cost"
        }
      }
    ],
    "DO" : [
      { "data_calc" : {
          "USING" : {
            "input_1" : [ "commerce-line-item:quantity" ],
            "op" : "*",
            "input_2" : [ "commerce-line-item:commerce-product:field-gocardless-shipping-cost:amount" ]
          },
          "PROVIDE" : { "result" : { "result" : "Calculation result" } }
        }
      },
      { "data_set" : { "data" : [ "shipping-cost-for-line-item" ], "value" : [ "result" ] } }
    ],"PROVIDES VARIABLES" : [ "shipping_cost_for_line_item" ]
    }}';

    $rules['commerce_gc_client_shipping_field_shipping_cost'] = '{ "commerce_gc_client_shipping_field_shipping_cost" : {
    "LABEL" : "Handle GoCardless shipping using field_shipping_cost",
    "PLUGIN" : "reaction rule",
    "OWNER" : "rules",
    "REQUIRES" : [ "rules", "commerce_gc_client", "commerce_shipping" ],
    "ON" : { "commerce_shipping_collect_rates" : [] },
    "DO" : [
      { "LOOP" : {
          "USING" : { "list" : [ "commerce-order:commerce-line-items" ] },
          "ITEM" : { "commerce_line_items" : "Commerce line items" },
          "DO" : [
            { "component_commerce_gc_client_calculate_field_shipping_cost" : {
                "USING" : { "commerce_line_item" : [ "commerce-line-items" ] },
                "PROVIDE" : { "shipping_cost_for_line_item" : { "shipping_cost_for_line_item" : "Shipping cost for line item" } }
              }
            },
            { "commerce_gc_client_rules_calculate_shipping_line_item_data" : {
                "commerce_line_item" : [ "commerce-line-items" ],
                "amount" : [ "shipping-cost-for-line-item" ]
              }
            }
          ]
        }
      }
    ]}}';

    $rules['commerce_gc_client_line_item_field_shipping_cost'] = '{ "commerce_gc_client_line_item_field_shipping_cost" : {
    "LABEL" : "Add shipping line item using field_shipping_cost",
    "PLUGIN" : "reaction rule",
    "OWNER" : "rules",
    "REQUIRES" : [ "rules", "commerce_line_item", "commerce_shipping" ],
    "ON" : { "commerce_shipping_calculate_rate" : [] },
    "DO" : [
      { "LOOP" : {
          "USING" : { "list" : [ "commerce-line-item:order:commerce-line-items" ] },
          "ITEM" : { "commerce_line_items" : "Commerce line items" },
          "DO" : [
            { "component_commerce_gc_client_calculate_field_shipping_cost" : {
                "USING" : { "commerce_line_item" : [ "commerce-line-items" ] },
                "PROVIDE" : { "shipping_cost_for_line_item" : { "shipping_cost_for_line_item" : "Shipping cost for line item" } }
              }
            },
            { "commerce_line_item_unit_price_add" : {
                "commerce_line_item" : [ "commerce_line_item" ],
                "amount" : [ "shipping-cost-for-line-item" ],
                "component_name" : "shipping",
                "round_mode" : "1"
              }
            }
          ]
        }
      }
    ]}}';
  }
  foreach ($rules as $key => $rule) {
    $configs[$key] = rules_import($rule);
  }
  return $configs;
}
